@@ -16,9 +16,11 @@
 # You should have received a copy of the GNU General Public License
 # along with  Hyperspy.  If not, see <http://www.gnu.org/licenses/>.
 
-import matplotlib.pyplot as plt
+import sys
 
+import matplotlib.pyplot as plt
 import numpy as np
+import scipy as sp
 import traits.api as t
 import traitsui.api as tu
 from traitsui.menu import OKButton, ApplyButton, CancelButton
@@ -29,13 +31,19 @@ from hyperspy.misc import utils
 from hyperspy import drawing
 from hyperspy.misc.interactive_ns import interactive_ns
 from hyperspy.gui.tools import (SpanSelectorInSpectrum, 
-    SpanSelectorInSpectrumHandler)
+    SpanSelectorInSpectrumHandler,OurFindButton, OurPreviousButton,
+    OurApplyButton)
 from hyperspy.misc.progressbar import progressbar
+import hyperspy.gui.messages as messages
 
 
 class BackgroundRemoval(SpanSelectorInSpectrum):
-    background_type = t.Enum('Power Law', 'Gaussian', 'Offset',
-    'Polynomial', default = 'Power Law')
+    background_type = t.Enum(
+        'Power Law',
+        'Gaussian',
+        'Offset',
+        'Polynomial',
+        default = 'Power Law')
     polynomial_order = t.Range(1,10)
     background_estimator = t.Instance(Component)
     bg_line_range = t.Enum('from_left_range', 'full', 'ss_range', 
@@ -48,7 +56,8 @@ class BackgroundRemoval(SpanSelectorInSpectrum):
                 'polynomial_order', 
                 visible_when = 'background_type == \'Polynomial\''),),
             buttons= [OKButton, CancelButton],
-            handler = SpanSelectorInSpectrumHandler)
+            handler = SpanSelectorInSpectrumHandler,
+            title = 'Background removal tool')
                  
     def __init__(self, signal):
         super(BackgroundRemoval, self).__init__(signal)
@@ -93,7 +102,7 @@ class BackgroundRemoval(SpanSelectorInSpectrum):
         self.bg_line = drawing.spectrum.SpectrumLine()
         self.bg_line.data_function = self.bg_to_plot
         self.bg_line.line_properties_helper('blue', 'line')
-        self.signal._plot.spectrum_plot.add_line(self.bg_line)
+        self.signal._plot.signal_plot.add_line(self.bg_line)
         self.bg_line.autoscale = False
         self.bg_line.plot()
         
@@ -155,9 +164,251 @@ class BackgroundRemoval(SpanSelectorInSpectrum):
             
         self.signal._replot()
         self.signal._plot.auto_update_plot = True
-    
+        
+class SpikesRemovalHandler(tu.Handler):
+    def close(self, info, is_ok):
+        # Removes the span selector from the plot
+        info.object.span_selector_switch(False)
+        return True
 
-       
+    def apply(self, info, *args, **kwargs):
+        """Handles the **Apply** button being clicked.
+
+        """
+        obj = info.object
+        obj.is_ok = True
+        if hasattr(obj, 'apply'):
+            obj.apply()
+        
+        return
+        
+    def find(self, info, *args, **kwargs):
+        """Handles the **Next** button being clicked.
+
+        """
+        obj = info.object
+        obj.is_ok = True
+        if hasattr(obj, 'find'):
+            obj.find()
+        return
+        
+    def back(self, info, *args, **kwargs):
+        """Handles the **Next** button being clicked.
+
+        """
+        obj = info.object
+        obj.is_ok = True
+        if hasattr(obj, 'find'):
+            obj.find(back=True)
+        return
+
+        
+class SpikesRemoval(SpanSelectorInSpectrum):
+    interpolator_kind = t.Enum(
+        'Linear',
+        'Spline',
+        default = 'Linear')
+    threshold = t.Float()
+    show_derivative_histogram = t.Button()
+    spline_order = t.Range(1,10, 3)
+    interpolator = None
+    default_spike_width = t.Int(5)
+    index = t.Int(0)
+    view = tu.View(tu.Group(
+        tu.Group(
+                 tu.Item('show_derivative_histogram', show_label=False),
+                 'threshold',
+                 show_border=True,),
+        tu.Group(
+            'interpolator_kind',
+            'default_spike_width',
+            tu.Group(
+                'spline_order', 
+                visible_when = 'interpolator_kind == \'Spline\''),
+            show_border=True,
+            label='Advanced settings'),
+            ),
+            buttons= [OKButton, OurPreviousButton, OurFindButton, OurApplyButton,
+                      ],
+            handler = SpikesRemovalHandler,
+            title = 'Spikes removal tool')
+                 
+    def __init__(self, signal,navigation_mask=None, signal_mask=None):
+        super(SpikesRemoval, self).__init__(signal)
+        self.interpolated_line = None
+        self.coordinates = [coordinate for coordinate in np.ndindex(
+                            tuple(signal.axes_manager.navigation_shape)) if (navigation_mask is None or not navigation_mask[coordinate])]
+        self.signal = signal
+        sys.setrecursionlimit(np.cumprod(self.signal.data.shape)[-1])
+        self.line = signal._plot.signal_plot.ax_lines[0]
+        self.ax = signal._plot.signal_plot.ax
+        signal._plot.auto_update_plot = False
+        signal.axes_manager.coordinates = self.coordinates[0]
+        self.threshold = 400
+        self.index = 0
+        self.argmax = None
+        self.kind = "linear"
+        self._temp_mask = np.zeros(self.signal().shape, dtype='bool')
+        self.signal_mask = signal_mask
+        self.navigation_mask = navigation_mask
+        
+    def _threshold_changed(self, old, new):
+        self.index = 0
+        self.update_plot()
+        
+    def _show_derivative_histogram_fired(self):
+        self.signal.spikes_diagnosis(signal_mask=self.signal_mask,
+                                     navigation_mask=self.navigation_mask)
+        
+    def detect_spike(self):
+        derivative = np.diff(self.signal())
+        if self.signal_mask is not None:
+            derivative[self.signal_mask[:-1]] = 0
+        if self.argmax is not None:
+            left, right = self.get_interpolation_range()
+            self._temp_mask[left:right] = True
+            derivative[self._temp_mask[:-1]] = 0
+        if abs(derivative.max()) >= self.threshold:
+            self.argmax = derivative.argmax()
+            return True
+        else:
+            return False
+
+    def find(self, back=False):
+        if (self.index == len(self.coordinates) - 1 and back is False) \
+        or (back is True and self.index == 0):
+            messages.information('End of dataset reached')
+            return
+        if self.interpolated_line is not None:
+            self.interpolated_line.close()
+            self.interpolated_line = None
+            self.reset_span_selector()
+        
+        if self.detect_spike() is False:
+            if back is False:
+                self.index += 1
+            else:
+                self.index -= 1
+            self.find(back=back)
+        else:
+            minimum = max(0,self.argmax - 50)
+            maximum = min(len(self.signal()) - 1, self.argmax + 50)
+            self.ax.set_xlim(
+                self.signal.axes_manager.axes[-1].index2value(minimum),
+                self.signal.axes_manager.axes[-1].index2value(maximum))
+            self.update_plot()
+            self.create_interpolation_line()
+
+    def update_plot(self):
+        if self.interpolated_line is not None:
+            self.interpolated_line.close()
+            self.interpolated_line = None
+        self.reset_span_selector()
+        self.update_spectrum_line()
+        self.signal._plot.pointer.update_patch_position()
+        
+    def update_spectrum_line(self):
+        self.line.auto_update = True
+        self.line.update()
+        self.line.auto_update = False
+        
+    def _index_changed(self, old, new):
+        self.signal.axes_manager.coordinates = self.coordinates[new]
+        self.argmax = None
+        self._temp_mask[:] = False
+        
+    def on_disabling_span_selector(self):
+        if self.interpolated_line is not None:
+            self.interpolated_line.close()
+            self.interpolated_line = None
+           
+    def _spline_order_changed(self, old, new):
+        self.kind = self.spline_order
+        self.span_selector_changed()
+            
+    def _interpolator_kind_changed(self, old, new):
+        if new == 'linear':
+            self.kind = new
+        else:
+            self.kind = self.spline_order
+        self.span_selector_changed()
+        
+    def _ss_left_value_changed(self, old, new):
+        self.span_selector_changed()
+        
+    def _ss_right_value_changed(self, old, new):
+        self.span_selector_changed()
+        
+    def create_interpolation_line(self):
+        self.interpolated_line = drawing.spectrum.SpectrumLine()
+        self.interpolated_line.data_function = \
+            self.get_interpolated_spectrum
+        self.interpolated_line.line_properties_helper('blue', 'line')
+        self.signal._plot.signal_plot.add_line(self.interpolated_line)
+        self.interpolated_line.autoscale = False
+        self.interpolated_line.plot()
+        
+    def get_interpolation_range(self):
+        axis = self.signal.axes_manager.signal_axes[0]
+        if self.ss_left_value == self.ss_right_value:
+            left = self.argmax - self.default_spike_width
+            right = self.argmax + self.default_spike_width
+        else:
+            left = axis.value2index(self.ss_left_value)
+            right = axis.value2index(self.ss_right_value)
+            
+        return left,right
+        
+        
+    def get_interpolated_spectrum(self, axes_manager=None):
+        data = self.signal().copy()
+        axis = self.signal.axes_manager.signal_axes[0]
+        left, right = self.get_interpolation_range()
+        if self.kind == 'linear':
+            pad = 1
+        else:
+            pad = 10
+        ileft = left - pad
+        iright = right + pad
+        ileft = np.clip(ileft, 0, len(data))
+        iright = np.clip(iright, 0, len(data))
+        left = np.clip(left, 0, len(data))
+        right = np.clip(right, 0, len(data))
+        x = np.hstack((axis.axis[ileft:left], axis.axis[right:iright]))
+        y = np.hstack((data[ileft:left], data[right:iright]))
+        if ileft == 0:
+            # Extrapolate to the left
+            data[left:right] = data[right + 1]
+            
+        elif iright == (len(data) - 1):
+            # Extrapolate to the right
+            data[left:right] = data[left - 1]
+            
+        else:
+            # Interpolate
+            intp = sp.interpolate.interp1d(x, y, kind=self.kind)
+            data[left:right] = intp(axis.axis[left:right])
+        
+        # Add noise
+        data = np.random.poisson(np.clip(data, 0, np.inf))
+        return data
+
+                      
+    def span_selector_changed(self):
+        if self.interpolated_line is None:
+            return
+        else:
+            self.interpolated_line.update()
+            
+    def apply(self):
+        self.signal()[:] = self.get_interpolated_spectrum()
+        self.update_spectrum_line()
+        self.interpolated_line.close()
+        self.interpolated_line = None
+        self.reset_span_selector()
+        self.find()
+        
+    
 #class EgertonPanel(t.HasTraits):
 #    define_background_window = t.Bool(False)
 #    bg_window_size_variation = t.Button()
@@ -212,7 +463,7 @@ class BackgroundRemoval(SpanSelectorInSpectrum):
 #        if new is True:
 #            self.span_selector = \
 #            drawing.widgets.ModifiableSpanSelector(
-#            self.signal.hse.spectrum_plot.ax,
+#            self.signal.hse.signal_plot.ax,
 #            onselect = self.store_current_spectrum_bg_parameters,
 #            onmove_callback = self.background_estimatorot_bg_removed_spectrum)
 #        elif self.span_selector is not None:
@@ -244,7 +495,7 @@ class BackgroundRemoval(SpanSelectorInSpectrum):
 #        if new is True:
 #            self.signal_span_selector = \
 #            drawing.widgets.ModifiableSpanSelector(
-#            self.signal.hse.spectrum_plot.ax, 
+#            self.signal.hse.signal_plot.ax, 
 #            onselect = self.store_current_spectrum_bg_parameters,
 #            onmove_callback = self.background_estimatorot_signal_map)
 #            self.signal_span_selector.rect.set_color('blue')
@@ -261,12 +512,12 @@ class BackgroundRemoval(SpanSelectorInSpectrum):
 #        if self.bg_line is not None:
 #            self.span_selector.ax.lines.remove(self.bg_line)
 #            self.span_selector.ax.lines.remove(self.signal_line)
-#        self.bg_line, = self.signal.hse.spectrum_plot.ax.plot(
+#        self.bg_line, = self.signal.hse.signal_plot.ax.plot(
 #        ea, self.background_estimator.function(ea), color = 'black')
-#        self.signal_line, = self.signal.hse.spectrum_plot.ax.plot(
+#        self.signal_line, = self.signal.hse.signal_plot.ax.plot(
 #        self.signal.energy_axis[iright:], self.signal()[iright:] - 
 #        self.background_estimator.function(self.signal.energy_axis[iright:]), color = 'black')
-#        self.signal.hse.spectrum_plot.ax.figure.canvas.draw()
+#        self.signal.hse.signal_plot.ax.figure.canvas.draw()
 
 #        
 #    def plot_signal_map(self, *args, **kwargs):
