@@ -28,6 +28,7 @@ import numpy as np
 
 from hyperspy.misc.utils_readfile import *
 from hyperspy import Release
+from hyperspy.misc.utils import DictionaryBrowser
 
 # Plugin characteristics
 # ----------------------
@@ -41,7 +42,7 @@ full_suport = False             #  but maybe True
 file_extensions = ['rpl','RPL']
 default_extension = 0
 # Writing capabilities
-writes = [(1,0), (1,1), (1,2), (2,1), (1,2),]
+writes = [(1,0), (1,1), (1,2), (2,0), (2,1),]
 # ----------------------
 
 # The format only support the followng data types
@@ -86,6 +87,11 @@ rpl_keys = {
     'height-scale' : float,
     'height-units' : unicode,
     'signal' : unicode,
+    # EELS Hyperspy keys
+    'collection-angle' : float,
+    # TEM Hyperespy keys
+    'convergence-angle' : float,
+    'beam-energy' : float,
     }
 
 def parse_ripple(fp):
@@ -97,7 +103,7 @@ def parse_ripple(fp):
         line = line.replace(' ', '')
         if line[:2] not in newline and line[0] != comment:
             line = line.strip('\r\n')
-            line = line.lower()
+            #line = line.lower()
             if comment in line:
                 line = line[:line.find(comment)]
             if not sep in line:
@@ -139,9 +145,26 @@ def parse_ripple(fp):
         raise IOError, err
     return rpl_info
 
-def read_raw(rpl_info, fp):
+def read_raw(rpl_info, fp, mmap_mode='c'):
     """Read the raw file object 'fp' based on the information given in the
     'rpl_info' dictionary.
+    
+    Parameters
+    ----------
+    rpl_info: dict
+        A dictionary containing the keywords as parsed by read_rpl
+    fp:
+    mmap_mode: {None, 'r+', 'r', 'w+', 'c'}, optional
+    If not None, then memory-map the file, using the given mode
+    (see `numpy.memmap`).  The mode has no effect for pickled or
+    zipped files.
+    A memory-mapped array is stored on disk, and not directly loaded
+    into memory.  However, it can be accessed and sliced like any
+    ndarray.  Memory mapping is especially useful for accessing
+    small fragments of large files without reading the entire file
+    into memory.
+    
+    
     """
     width = rpl_info['width']
     height = rpl_info['height']
@@ -172,9 +195,10 @@ def read_raw(rpl_info, fp):
     data_type = np.dtype(data_type)
     data_type = data_type.newbyteorder(endian)
 
-    data = read_data_array(fp,
-                           byte_address=offset,
-                           data_type=data_type)
+    data = np.memmap(fp,
+                     offset=offset,
+                     dtype=data_type,
+                     mode=mmap_mode)
 
     if record_by == 'vector':   # spectral image
         size = (height, width, depth)
@@ -187,8 +211,8 @@ def read_raw(rpl_info, fp):
         data = data.reshape(size)
     return data
 
-def file_reader(filename, rpl_info=None, encoding="latin-1", 
-                *args,**kwds):
+def file_reader(filename, rpl_info=None, encoding="latin-1",
+                mmap_mode='c', *args,**kwds):
     """Parses a Lispix (http://www.nist.gov/lispix/) ripple (.rpl) file
     and reads the data from the corresponding raw (.raw) file;
     or, read a raw file if the dictionary rpl_info is provided.
@@ -246,6 +270,9 @@ def file_reader(filename, rpl_info=None, encoding="latin-1",
       height-units          str      # row units, usually nm
       height-name      str           # Name of the magnitude stored as height
       signal            str        # Name of the signal stored, e.g. HAADF
+      convergence-angle float   # TEM convergence angle in mrad
+      collection-angle  float   # EELS spectrometer collection angle in mrad
+      beam-energy       float   # TEM beam energy in keV
 
     NOTES
 
@@ -297,7 +324,7 @@ def file_reader(filename, rpl_info=None, encoding="latin-1",
     if not rawfname:
         raise IOError, 'RAW file "%s" does not exists' % rawfname
     else:
-        data = read_raw(rpl_info, rawfname)
+        data = read_raw(rpl_info, rawfname, mmap_mode=mmap_mode)
 
     if rpl_info['record-by'] == 'vector':
         print 'Loading as spectrum'
@@ -327,7 +354,7 @@ def file_reader(filename, rpl_info=None, encoding="latin-1",
 
     if 'signal' not in rpl_info:
         rpl_info['signal'] = None
-
+        
     if rpl_info.has_key('detector-peak-width-ev'):
         original_parameters['detector-peak-width-ev'] = \
         rpl_info['detector-peak-width-ev']
@@ -371,7 +398,23 @@ def file_reader(filename, rpl_info=None, encoding="latin-1",
 
     if rpl_info.has_key('height-name'):
         names[iheight] = rpl_info['height-name']
-
+        
+    
+    mp = DictionaryBrowser({
+			'record_by': record_by,
+			'original_filename': filename,
+            'signal_type': rpl_info['signal'],
+			})
+            
+    if 'convergence-angle' in rpl_info:
+        mp.set_item('TEM.convergence_angle', 
+            rpl_info['convergence-angle'])
+    if 'collection-angle' in rpl_info:
+        mp.set_item('TEM.EELS.collection_angle', 
+            rpl_info['collection-angle'])
+    if 'beam-energy' in rpl_info:
+        mp.set_item('TEM.beam_energy', 
+            rpl_info['beam-energy'])
     axes = []
     index_in_array = 0
     for i in xrange(3):
@@ -389,11 +432,7 @@ def file_reader(filename, rpl_info=None, encoding="latin-1",
     dictionary = {
         'data': data.squeeze(),
         'axes': axes,
-        'mapped_parameters': {
-			'record_by': record_by,
-			'original_filename': filename,
-                        'signal_type': rpl_info['signal'],
-			},
+        'mapped_parameters': mp.as_dictionary(),
         'original_parameters': rpl_info
         }
     return [dictionary, ]
@@ -422,30 +461,30 @@ def file_writer(filename, signal, encoding='latin-1', *args, **kwds):
     byte_order = endianess2rpl[dc.dtype.byteorder.replace('|', '=')]
     offset = 0
     if hasattr(signal.mapped_parameters,'signal_type'):
-        signal_type=signal.mapped_parameters.signal_type
+        signal_type = signal.mapped_parameters.signal_type
     else:
         signal_type=None
     if signal.axes_manager.signal_dimension == 1:
         record_by = 'vector'
-        depth_axis = signal.axes_manager._slicing_axes[0]
+        depth_axis = signal.axes_manager.signal_axes[0]
         ev_per_chan = int(round(depth_axis.scale))
         if dimension == 3:
-            width_axis = signal.axes_manager._non_slicing_axes[1]
-            height_axis = signal.axes_manager._non_slicing_axes[0]
+            width_axis = signal.axes_manager.navigation_axes[1]
+            height_axis = signal.axes_manager.navigation_axes[0]
             depth, width, height = \
                             depth_axis.size, width_axis.size, height_axis.size
         elif dimension == 2:
-            width_axis = signal.axes_manager._non_slicing_axes[0]
+            width_axis = signal.axes_manager.navigation_axes[0]
             depth, width, height = depth_axis.size, width_axis.size,1
         elif dimension == 1:
             record_by == 'dont-care'
             depth, width, height = depth_axis.size, 1, 1
 
     elif signal.axes_manager.signal_dimension == 2:
-        width_axis = signal.axes_manager._slicing_axes[1]
-        height_axis = signal.axes_manager._slicing_axes[0]
+        width_axis = signal.axes_manager.signal_axes[1]
+        height_axis = signal.axes_manager.signal_axes[0]
         if dimension == 3:
-            depth_axis = signal.axes_manager._non_slicing_axes[0]
+            depth_axis = signal.axes_manager.navigation_axes[0]
             record_by = 'image'
             depth, width, height =  \
                             depth_axis.size, width_axis.size, height_axis.size
@@ -476,10 +515,22 @@ def file_writer(filename, signal, encoding='latin-1', *args, **kwds):
     keys = ['depth', 'height', 'width']
     for key in keys:
         if eval(key) > 1:
-            keys_dictionary['%s-scale' % key] = eval('%s_axis.scale' % key)
-            keys_dictionary['%s-origin' % key] = eval('%s_axis.offset' % key)
-            keys_dictionary['%s-units' % key] = eval('%s_axis.units' % key)
-            keys_dictionary['%s-name' % key] = eval('%s_axis.name' % key)
+            keys_dictionary['%s-scale' % key] = eval(
+                '%s_axis.scale' % key)
+            keys_dictionary['%s-origin' % key] = eval(
+                '%s_axis.offset' % key)
+            keys_dictionary['%s-units' % key] = eval(
+                '%s_axis.units' % key)
+            keys_dictionary['%s-name' % key] = eval(
+                '%s_axis.name' % key)
+            
+    mp = signal.mapped_parameters
+    if mp.has_item('TEM.beam_energy'):
+        keys_dictionary['beam-energy'] = mp.TEM.beam_energy
+    if mp.has_item('TEM.convergence_angle'):
+        keys_dictionary['convergence-angle'] = mp.TEM.convergence_angle
+    if mp.has_item('TEM.EELS.collection_angle'):
+        keys_dictionary['collection-angle'] = mp.TEM.EELS.collection_angle
 
     write_rpl(filename, keys_dictionary, encoding)
     write_raw(filename, signal, record_by)
@@ -514,16 +565,16 @@ def write_raw(filename, signal, record_by):
     if len(dshape) == 3:
         if record_by == 'vector':
             np.rollaxis(
-                data, signal.axes_manager._slicing_axes[0].index_in_array, 3
+                data, signal.axes_manager.signal_axes[0].index_in_array, 3
                         ).ravel().tofile(filename)
         elif record_by == 'image':
             data = np.rollaxis(
-                data, signal.axes_manager._non_slicing_axes[0].index_in_array, 0
+                data, signal.axes_manager.navigation_axes[0].index_in_array, 0
                         ).ravel().tofile(filename)
     elif len(dshape) == 2:
         if record_by == 'vector':
             np.rollaxis(
-                data, signal.axes_manager._slicing_axes[0].index_in_array, 2
+                data, signal.axes_manager.signal_axes[0].index_in_array, 2
                         ).ravel().tofile(filename)
         elif record_by in ('image', 'dont-care'):
             data.ravel().tofile(filename)
