@@ -30,12 +30,9 @@ try:
     mdp_installed = True
 except:
     mdp_installed = False
-try:
-    import sklearn.decomposition
-    sklearn_installed = True
-except:
-    sklearn_installed = False
+    
 
+from hyperspy.misc.import_sklearn import *
 from hyperspy.misc import utils
 from hyperspy.learn.svd_pca import svd_pca
 from hyperspy.learn.mlpca import mlpca
@@ -68,10 +65,19 @@ class MVA():
             self.learning_results = LearningResults()
 
     @do_not_replot
-    def decomposition(self, normalize_poissonian_noise=False,
-    algorithm = 'svd', output_dimension=None, centre = None,
-    auto_transpose = True, navigation_mask=None, signal_mask=None,
-    var_array=None, var_func=None, polyfit=None, reproject=None, **kwargs):
+    def decomposition(self,
+        normalize_poissonian_noise=False,
+        algorithm = 'svd',
+        output_dimension=None,
+        centre=None,
+        auto_transpose=True,
+        navigation_mask=None,
+        signal_mask=None,
+        var_array=None,
+        var_func=None,
+        polyfit=None,
+        reproject=None,
+        **kwargs):
         """Decomposition with a choice of algorithms
 
         The results are stored in self.learning_results
@@ -98,8 +104,12 @@ class MVA():
             Only has effect when using the svd of fast_svd algorithms.
             
         navigation_mask : boolean numpy array
+            The navigation locations marked as True are not used in the 
+            decompostion.
         
         signal_mask : boolean numpy array
+            The signal locations marked as True are not used in the 
+            decomposition.
             
         var_array : numpy array
             Array of variance for the maximum likelihood PCA algorithm
@@ -147,231 +157,252 @@ class MVA():
         # Apply pre-treatments
         # Transform the data in a line spectrum
         self._unfolded4decomposition = self.unfold_if_multidim()
-        if hasattr(navigation_mask, 'ravel'):
-            navigation_mask = navigation_mask.ravel()
+        try:
+            if hasattr(navigation_mask, 'ravel'):
+                navigation_mask = navigation_mask.ravel()
 
-        if hasattr(signal_mask, 'ravel'):
-            signal_mask = signal_mask.ravel()
+            if hasattr(signal_mask, 'ravel'):
+                signal_mask = signal_mask.ravel()
 
-        # Normalize the poissonian noise
-        # TODO this function can change the masks and this can cause
-        # problems when reprojecting
-        if normalize_poissonian_noise is True:
-            if reproject is None:
-                navigation_mask, signal_mask = \
-                    self.normalize_poissonian_noise(
-                                            navigation_mask=navigation_mask,
-                                            signal_mask=signal_mask,
-                                            return_masks = True)
-            elif reproject == 'both':
-                _, _ = \
-                    self.normalize_poissonian_noise(return_masks = True)  
-            elif reproject == 'navigation':
-                _, signal_mask = \
-                    self.normalize_poissonian_noise(return_masks = True,
-                                                    signal_mask=signal_mask,) 
-            elif reproject == 'signal':
-                navigation_mask, _ = \
-                    self.normalize_poissonian_noise(return_masks = True,
-                                            navigation_mask=navigation_mask,)         
+            # Normalize the poissonian noise
+            # TODO this function can change the masks and this can cause
+            # problems when reprojecting
+            if normalize_poissonian_noise is True:
+                self.normalize_poissonian_noise(
+                                        navigation_mask=navigation_mask,
+                                        signal_mask=signal_mask,)
+            messages.information('Performing decomposition analysis')
+
+            dc = self.data
+            #set the output target (peak results or not?)
+            target = self.learning_results
             
-        messages.information('Performing decomposition analysis')
-
-        dc = self.data
-            
-        #set the output target (peak results or not?)
-        target = self.learning_results
-        
-        # Transform the None masks in slices to get the right behaviour
-        if navigation_mask is None:
-            navigation_mask = slice(None)
-        if signal_mask is None:
-            signal_mask = slice(None)
-        
-        # Reset the explained_variance which is not set by all the algorithms
-        explained_variance = None
-        explained_variance_ratio = None
-        mean = None
-        
-        if algorithm == 'svd':
-            factors, loadings, explained_variance, mean = svd_pca(
-                dc[:,signal_mask][navigation_mask,:], centre = centre,
-                auto_transpose = auto_transpose)
-
-        elif algorithm == 'fast_svd':
-            factors, loadings, explained_variance, mean = svd_pca(
-                dc[:,signal_mask][navigation_mask,:],
-            fast = True, output_dimension = output_dimension, centre = centre,
-                auto_transpose = auto_transpose)
-
-        elif algorithm == 'sklearn_pca':
-            if sklearn_installed is False:
-                raise ImportError(
-                'sklearn is not installed. Nothing done')
-            sk = sklearn.decomposition.PCA(**kwargs)
-            sk.n_components = output_dimension
-            loadings = sk.fit_transform((dc[:,signal_mask][navigation_mask,:]))
-            factors = sk.components_.T
-            explained_variance = sk.explained_variance_
-            mean = sk.mean_
-            centre = 'trials'   
-
-        elif algorithm == 'nmf':
-            if sklearn_installed is False:
-                raise ImportError(
-                'sklearn is not installed. Nothing done')
-            sk = sklearn.decomposition.NMF(**kwargs)
-            sk.n_components = output_dimension
-            loadings = sk.fit_transform((dc[:,signal_mask][navigation_mask,:]))
-            factors = sk.components_.T
-            
-        elif algorithm == 'sparse_pca':
-            if sklearn_installed is False:
-                raise ImportError(
-                'sklearn is not installed. Nothing done')
-            sk = sklearn.decomposition.SparsePCA(output_dimension, **kwargs)
-            loadings = sk.fit_transform(dc[:,signal_mask][navigation_mask,:])
-            factors = sk.components_.T
-            
-        elif algorithm == 'mini_batch_sparse_pca':
-            if sklearn_installed is False:
-                raise ImportError(
-                'sklearn is not installed. Nothing done')
-            sk = sklearn.decomposition.MiniBatchSparsePCA(output_dimension,
-                                                            **kwargs)
-            loadings = sk.fit_transform(dc[:,signal_mask][navigation_mask,:])
-            factors = sk.components_.T
-
-        elif algorithm == 'mlpca' or algorithm == 'fast_mlpca':
-            print "Performing the MLPCA training"
-            if output_dimension is None:
-                messages.warning_exit(
-                "For MLPCA it is mandatory to define the output_dimension")
-            if var_array is None and var_func is None:
-                messages.information('No variance array provided.'
-                'Supposing poissonian data')
-                var_array = dc[:,signal_mask][navigation_mask,:]
-
-            if var_array is not None and var_func is not None:
-                messages.warning_exit(
-                "You have defined both the var_func and var_array keywords"
-                "Please, define just one of them")
-            if var_func is not None:
-                if hasattr(var_func, '__call__'):
-                    var_array = var_func(dc[signal_mask,...][:,navigation_mask])
-                else:
-                    try:
-                        var_array = np.polyval(polyfit,dc[signal_mask,
-                        navigation_mask])
-                    except:
-                        messages.warning_exit(
-                        'var_func must be either a function or an array'
-                        'defining the coefficients of a polynom')
-            if algorithm == 'mlpca':
-                fast = False
+            # Transform the None masks in slices to get the right behaviour
+            if navigation_mask is None:
+                navigation_mask = slice(None)
             else:
-                fast = True
-            U,S,V,Sobj, ErrFlag = mlpca(
-                dc[:,signal_mask][navigation_mask,:],
-                var_array, output_dimension, fast = fast)
-            loadings = U * S
-            factors = V
-            explained_variance_ratio = S ** 2 / Sobj
-            explained_variance = S ** 2 / len(factors)
-        else:
-            raise ValueError('Algorithm not recognised. '
-                                 'Nothing done')
-
-        # We must calculate the ratio here because otherwise the sum information
-        # can be lost if the user call crop_decomposition_dimension
-        if explained_variance is not None and explained_variance_ratio is None:
-            explained_variance_ratio = \
-                explained_variance / explained_variance.sum()
+                navigation_mask = ~navigation_mask
+            if signal_mask is None:
+                signal_mask = slice(None)
+            else:
+                signal_mask = ~signal_mask
                 
-        # Store the results in learning_results
-        target.factors = factors
-        target.loadings = loadings
-        target.explained_variance = explained_variance
-        target.explained_variance_ratio = explained_variance_ratio
-        target.decomposition_algorithm = algorithm
-        target.poissonian_noise_normalized = \
-            normalize_poissonian_noise
-        target.output_dimension = output_dimension
-        target.unfolded = self._unfolded4decomposition
-        target.centre = centre
-        target.mean = mean
-        
-
-        if output_dimension and factors.shape[1] != output_dimension:
-            target.crop_decomposition_dimension(output_dimension)
-        
-        # Delete the unmixing information, because it'll refer to a previous
-        # decompositions
-        target.unmixing_matrix = None
-        target.bss_algorithm = None
-
-        if self._unfolded4decomposition is True:
-            target.original_shape = self._shape_before_unfolding
-
-        # Reproject
-        if mean is None:
-            mean = 0
-        if reproject in ('navigation', 'both'):
-            if algorithm not in ('nmf', 'sparse_pca', 'mini_batch_sparse_pca'):
-                loadings_ = np.dot(dc[:,signal_mask] - mean, factors)
-            else:
-                loadings_ = sk.transform(dc[:,signal_mask])
-            target.loadings = loadings_
-        if reproject in ('signal', 'both'):
-            if algorithm not in ('nmf', 'sparse_pca', 'mini_batch_sparse_pca'):
-                factors = np.dot(np.linalg.pinv(loadings), 
-                                 dc[navigation_mask,:] - mean).T
-                target.factors = factors
-            else:
-                messages.information("Reprojecting the signal is not yet "
-                                     "supported for this algorithm")
-                if reproject == 'both':
-                    reproject = 'signal'
-                else:
-                    reproject = None
-        
-        # Rescale the results if the noise was normalized
-        if normalize_poissonian_noise is True:
-            target.factors[:] *= self._root_bH.T
-            target.loadings[:] *= self._root_aG
+            # WARNING: signal_mask and navigation_mask values are now their
+            # negaties i.e. True -> False and viceversa. However, the 
+            # stored value (at the end of the method) coincides with the 
+            # input masks
             
-        # Set the pixels that were not processed to nan
-        if not isinstance(signal_mask, slice):
-            target.signal_mask = signal_mask
-            if reproject not in ('both', 'signal'):
-                factors = np.zeros((dc.shape[-1], target.factors.shape[1]))
-                factors[signal_mask == True,:] = target.factors
-                factors[signal_mask == False,:] = np.nan
-                target.factors = factors
-        if not isinstance(navigation_mask, slice):
-            target.navigation_mask = navigation_mask
-            if reproject not in ('both', 'navigation'):
-                loadings = np.zeros((dc.shape[0], target.loadings.shape[1]))
-                loadings[navigation_mask == True,:] = target.loadings
-                loadings[navigation_mask == False,:] = np.nan
-                target.loadings = loadings
+            # Reset the explained_variance which is not set by all the 
+            # algorithms
+            explained_variance = None
+            explained_variance_ratio = None
+            mean = None
+            
+            if algorithm == 'svd':
+                factors, loadings, explained_variance, mean = svd_pca(
+                    dc[:,signal_mask][navigation_mask,:], centre = centre,
+                    auto_transpose = auto_transpose)
 
-        #undo any pre-treatments
-        self.undo_treatments()
-        
-        if self._unfolded4decomposition is True:
-            self.fold()
-            self._unfolded4decomposition is False
+            elif algorithm == 'fast_svd':
+                factors, loadings, explained_variance, mean = svd_pca(
+                    dc[:,signal_mask][navigation_mask,:],
+                    fast=True,
+                    output_dimension=output_dimension,
+                    centre=centre,
+                    auto_transpose=auto_transpose)
+
+            elif algorithm == 'sklearn_pca':
+                if sklearn_installed is False:
+                    raise ImportError(
+                    'sklearn is not installed. Nothing done')
+                sk = sklearn.decomposition.PCA(**kwargs)
+                sk.n_components = output_dimension
+                loadings = sk.fit_transform((
+                    dc[:,signal_mask][navigation_mask,:]))
+                factors = sk.components_.T
+                explained_variance = sk.explained_variance_
+                mean = sk.mean_
+                centre = 'trials'   
+
+            elif algorithm == 'nmf':
+                if sklearn_installed is False:
+                    raise ImportError(
+                    'sklearn is not installed. Nothing done')
+                sk = sklearn.decomposition.NMF(**kwargs)
+                sk.n_components = output_dimension
+                loadings = sk.fit_transform((
+                    dc[:,signal_mask][navigation_mask,:]))
+                factors = sk.components_.T
+                
+            elif algorithm == 'sparse_pca':
+                if sklearn_installed is False:
+                    raise ImportError(
+                    'sklearn is not installed. Nothing done')
+                sk = sklearn.decomposition.SparsePCA(
+                    output_dimension, **kwargs)
+                loadings = sk.fit_transform(
+                    dc[:,signal_mask][navigation_mask,:])
+                factors = sk.components_.T
+                
+            elif algorithm == 'mini_batch_sparse_pca':
+                if sklearn_installed is False:
+                    raise ImportError(
+                    'sklearn is not installed. Nothing done')
+                sk = sklearn.decomposition.MiniBatchSparsePCA(
+                    output_dimension, **kwargs)
+                loadings = sk.fit_transform(
+                    dc[:,signal_mask][navigation_mask,:])
+                factors = sk.components_.T
+
+            elif algorithm == 'mlpca' or algorithm == 'fast_mlpca':
+                print "Performing the MLPCA training"
+                if output_dimension is None:
+                    messages.warning_exit(
+                    "For MLPCA it is mandatory to define the "
+                    "output_dimension")
+                if var_array is None and var_func is None:
+                    messages.information('No variance array provided.'
+                    'Supposing poissonian data')
+                    var_array = dc[:,signal_mask][navigation_mask,:]
+
+                if var_array is not None and var_func is not None:
+                    messages.warning_exit(
+                    "You have defined both the var_func and var_array "
+                    "keywords."
+                    "Please, define just one of them")
+                if var_func is not None:
+                    if hasattr(var_func, '__call__'):
+                        var_array = var_func(
+                            dc[signal_mask,...][:,navigation_mask])
+                    else:
+                        try:
+                            var_array = np.polyval(polyfit,dc[signal_mask,
+                            navigation_mask])
+                        except:
+                            messages.warning_exit(
+                            'var_func must be either a function or an array'
+                            'defining the coefficients of a polynom')
+                if algorithm == 'mlpca':
+                    fast = False
+                else:
+                    fast = True
+                U,S,V,Sobj, ErrFlag = mlpca(
+                    dc[:,signal_mask][navigation_mask,:],
+                    var_array, output_dimension, fast = fast)
+                loadings = U * S
+                factors = V
+                explained_variance_ratio = S ** 2 / Sobj
+                explained_variance = S ** 2 / len(factors)
+            else:
+                raise ValueError('Algorithm not recognised. '
+                                     'Nothing done')
+
+            # We must calculate the ratio here because otherwise the sum 
+            # information can be lost if the user call 
+            # crop_decomposition_dimension
+            if explained_variance is not None and \
+            explained_variance_ratio is None:
+                explained_variance_ratio = \
+                    explained_variance / explained_variance.sum()
+                    
+            # Store the results in learning_results
+            target.factors = factors
+            target.loadings = loadings
+            target.explained_variance = explained_variance
+            target.explained_variance_ratio = explained_variance_ratio
+            target.decomposition_algorithm = algorithm
+            target.poissonian_noise_normalized = \
+                normalize_poissonian_noise
+            target.output_dimension = output_dimension
+            target.unfolded = self._unfolded4decomposition
+            target.centre = centre
+            target.mean = mean
+            
+
+            if output_dimension and factors.shape[1] != output_dimension:
+                target.crop_decomposition_dimension(output_dimension)
+            
+            # Delete the unmixing information, because it'll refer to a previous
+            # decompositions
+            target.unmixing_matrix = None
+            target.bss_algorithm = None
+
+            if self._unfolded4decomposition is True:
+                target.original_shape = self._shape_before_unfolding
+
+            # Reproject
+            if mean is None:
+                mean = 0
+            if reproject in ('navigation', 'both'):
+                if algorithm not in ('nmf', 'sparse_pca', 
+                                      'mini_batch_sparse_pca'):
+                    loadings_ = np.dot(dc[:,signal_mask] - mean, factors)
+                else:
+                    loadings_ = sk.transform(dc[:,signal_mask])
+                target.loadings = loadings_
+            if reproject in ('signal', 'both'):
+                if algorithm not in ('nmf', 'sparse_pca',
+                                      'mini_batch_sparse_pca'):
+                    factors = np.dot(np.linalg.pinv(loadings), 
+                                     dc[navigation_mask,:] - mean).T
+                    target.factors = factors
+                else:
+                    messages.information("Reprojecting the signal is not yet "
+                                         "supported for this algorithm")
+                    if reproject == 'both':
+                        reproject = 'signal'
+                    else:
+                        reproject = None
+            
+            # Rescale the results if the noise was normalized
+            if normalize_poissonian_noise is True:
+                target.factors[:] *= self._root_bH.T
+                target.loadings[:] *= self._root_aG
+                
+            # Set the pixels that were not processed to nan
+            if not isinstance(signal_mask, slice):
+                # Store the (inverted, as inputed) signal mask 
+                target.signal_mask = ~signal_mask.reshape(
+                    self.axes_manager.signal_shape)
+                if reproject not in ('both', 'signal'):
+                    factors = np.zeros((dc.shape[-1], target.factors.shape[1]))
+                    factors[signal_mask == True,:] = target.factors
+                    factors[signal_mask == False,:] = np.nan
+                    target.factors = factors
+            if not isinstance(navigation_mask, slice):
+                # Store the (inverted, as inputed) navigation mask
+                target.navigation_mask = ~navigation_mask.reshape(
+                    self.axes_manager.navigation_shape)
+                if reproject not in ('both', 'navigation'):
+                    loadings = np.zeros((dc.shape[0], target.loadings.shape[1]))
+                    loadings[navigation_mask == True,:] = target.loadings
+                    loadings[navigation_mask == False,:] = np.nan
+                    target.loadings = loadings
+        finally:
+            #undo any pre-treatments
+            self.undo_treatments()
+            
+            if self._unfolded4decomposition is True:
+                self.fold()
+                self._unfolded4decomposition is False
     
     def get_factors_as_spectrum(self):
         from hyperspy.signals.spectrum import Spectrum
-        return Spectrum({'data' : self.learning_results.factors.T})
+        return Spectrum(
+            {'data' : self.learning_results.factors.T.copy()})
     
-    def blind_source_separation(
-        self, number_of_components=None, algorithm='CuBICA', diff_order=1,
-        factors=None, comp_list=None, mask=None, 
-        on_loadings=False,pretreatment = None, **kwargs):
-        """Blind source separation (BSS) on the result on the decomposition.
+    def blind_source_separation(self,
+                                number_of_components=None,
+                                algorithm='sklearn_fastica',
+                                diff_order=1,
+                                factors=None,
+                                comp_list=None,
+                                mask=None, 
+                                on_loadings=False,
+                                pretreatment=None,
+                                **kwargs):
+        """Blind source separation (BSS) on the result on the 
+        decomposition.
 
         Available algorithms: FastICA, JADE, CuBICA, and TDSEP
 
@@ -381,23 +412,31 @@ class MVA():
             number of principal components to pass to the BSS algorithm
         algorithm : {FastICA, JADE, CuBICA, TDSEP}
         diff_order : int
-        factors : numpy array
-            externally provided components
+            Sometimes it is convenient to perform the BSS on the derivative 
+            of the signal. If diff_order is 0, the signal is not differentiated.
+        factors : numpy.array
+            Factors to decompose. If None, the BSS is performed on the result
+            of a previous decomposition.
         comp_list : boolen numpy array
-            choose the components to use by the boolean list. It permits to
-            choose non contiguous components.
-        mask : numpy boolean array with the same dimension as the PC
-            If not None, only the selected channels will be used by the
-            algorithm.
+            choose the components to use by the boolean list. It permits
+             to choose non contiguous components.
+        mask : numpy boolean array with the same dimension as the signal
+            If not None, the signal locations marked as True (masked) will 
+            not be passed to the BSS algorithm.
+        on_loadings : bool
+            If True, perform the BSS on the loadings of a previous 
+            decomposition. If False, performs it on the factors.
         pretreatment: dict
         
-        Any extra parameter is passed to the ICA algorithm
+        **kwargs : extra key word arguments
+            Any keyword arguments are passed to the BSS algorithm.
+        
         """
-        target=self.learning_results
-
+        target=self.learning_results                
         if not hasattr(target, 'factors') or target.factors==None:
-            self.decomposition()
-
+            raise AttributeError(
+                'A decomposition must be performed before blind '
+                'source seperation or factors must be provided.')
         else:
             if factors is None:
                 if on_loadings:
@@ -417,46 +456,52 @@ class MVA():
                     bool_index[ifactors] = True
                 number_of_components = len(comp_list)
             factors = factors[:,bool_index]
-            if diff_order > 0 and pretreatment is None:
-                factors = np.diff(factors, diff_order, axis = 0)
+                    
             if pretreatment is not None:
                 from hyperspy.signals.spectrum import Spectrum
                 sfactors = Spectrum({'data' : factors.T})
                 if pretreatment['algorithm'] == 'savitzky_golay':
                     sfactors.smooth_savitzky_golay(
-                        number_of_points = pretreatment['number_of_points'],
-                        polynomial_order = pretreatment['polynomial_order'],
+                        number_of_points=pretreatment[
+                                'number_of_points'],
+                        polynomial_order=pretreatment[
+                                'polynomial_order'],
                         differential_order = diff_order)
                 if pretreatment['algorithm'] == 'tv':
                     sfactors.smooth_tv(
-                        pretreatment_parameter= pretreatment[
-                            'pretreatment_parameter'],
+                        smoothing_parameter= pretreatment[
+                            'smoothing_parameter'],
                         differential_order = diff_order)
                 factors = sfactors.data.T
                 if pretreatment['algorithm'] == 'butter':
                     b, a = sp.signal.butter(pretreatment['order'],
                         pretreatment['cutoff'], pretreatment['type'])
                     for i in range(factors.shape[1]):
-                        factors[:,i] = sp.signal.filtfilt(b, a, factors[:,i])
-            
+                        factors[:,i] = sp.signal.filtfilt(b,a,
+                            factors[:,i])
+            elif diff_order > 0:
+                factors = np.diff(factors, diff_order, axis=0)
+                    
             if mask is not None:
-                factors = factors[mask.ravel(), :]
+                factors = factors[~mask]
 
-            # first centers and scales data
+            # first center and scale the data
             factors,invsqcovmat = centering_and_whitening(factors)
             if algorithm == 'orthomax':
                 _, unmixing_matrix = utils.orthomax(factors, **kwargs)
                 unmixing_matrix = unmixing_matrix.T
             
             elif algorithm == 'sklearn_fastica':
-                if sklearn_installed is False:
-                    raise ImportError(
-                    'sklearn is not installed. Nothing done')
-                target.bss_node = sklearn.decomposition.FastICA(**kwargs)
+                #if sklearn_installed is False:
+                    #raise ImportError(
+                    #'sklearn is not installed. Nothing done')
+                if 'tol' not in kwargs:
+                    kwargs['tol'] = 1e-10
+                target.bss_node = FastICA(
+                    **kwargs)
                 target.bss_node.whiten = False
                 target.bss_node.fit(factors)
                 unmixing_matrix = target.bss_node.unmixing_matrix_
-            
             else:
                 if mdp_installed is False:
                     raise ImportError(
@@ -469,11 +514,51 @@ class MVA():
                 target.bss_node.train(factors)
                 unmixing_matrix = target.bss_node.get_recmatrix()
 
-            target.unmixing_matrix = np.dot(unmixing_matrix, invsqcovmat)
+            target.unmixing_matrix = np.dot(unmixing_matrix,invsqcovmat)
             self._unmix_factors(target)
             self._unmix_loadings(target)
             self._auto_reverse_bss_component(target)
             target.bss_algorithm = algorithm
+            
+    def normalize_factors(self, which='bss',by='area', sort=True):
+        """Normalises the factors and modifies the loadings 
+        accordingly
+        
+        Parameters
+        ----------
+        which : 'bss' | 'decomposition'
+        by : 'max' | 'area'
+        sort : bool
+        
+        """
+        if which == 'bss':
+            factors = self.learning_results.bss_factors
+            loadings = self.learning_results.bss_loadings
+            if factors is None:
+                raise UserWarning("This method can only be used after "
+                    "a blind source separation operation")
+        elif which == 'decomposition':
+            factors = self.learning_results.factors
+            loadings = self.learning_results.loadings
+            if factors is None:
+                raise UserWarning("This method can only be used after"
+                    "a decomposition operation")
+        else:
+            raise ValueError("what must be bss or decomposition")
+        
+        if by == 'max':
+            by = np.max
+        elif by == 'area':
+            by = np.sum
+        else:
+            raise ValueError("by must be max or mean")
+            
+        factors /= by(factors,0)
+        loadings *= by(factors,0)
+        sorting_indexes = np.argsort(loadings.max(0))
+        factors[:] = factors[:,sorting_indexes]
+        loadings[:] = loadings[:,sorting_indexes]
+        loadings[:] = loadings[:,sorting_indexes]
 
     def reverse_bss_component(self, component_number):
         """Reverse the independent component
@@ -650,6 +735,12 @@ class MVA():
             created
         label: str
             An optional label for the legend
+            
+        Returns
+        -------
+        The axe of the plot, that can be passed to the method again in
+        a future call using the ax attribute
+        
         """
         target = self.learning_results
         if target.explained_variance_ratio is None:
@@ -661,7 +752,8 @@ class MVA():
         if ax is None:
             fig = plt.figure()
             ax = fig.add_subplot(111)
-        ax.plot(range(n), target.explained_variance_ratio[:n], 'o', label=label)
+        ax.plot(range(n), target.explained_variance_ratio[:n], 'o',
+            label=label)
         if log is True:
             ax.semilogy()
         ax.set_ylabel('Explained variance ratio')
@@ -671,8 +763,8 @@ class MVA():
         return ax
 
     def plot_cumulative_explained_variance_ratio(self,n=50):
-        """Plot the principal components explained variance up to the given
-        number
+        """Plot the principal components explained variance up to the 
+        given number
 
         Parameters
         ----------
@@ -682,7 +774,7 @@ class MVA():
         if n > target.explained_variance.shape[0]:
             n=target.explained_variance.shape[0]
         cumu = np.cumsum(target.explained_variance) / np.sum(
-                                                     target.explained_variance)
+                                            target.explained_variance)
         fig = plt.figure()
         ax = fig.add_subplot(111)
         ax.scatter(range(n), cumu[:n])
@@ -692,11 +784,11 @@ class MVA():
         plt.show()
         return ax
 
-    def normalize_poissonian_noise(self, navigation_mask = None,
-                                   signal_mask = None, return_masks = False):
+    def normalize_poissonian_noise(self, navigation_mask=None,
+                                   signal_mask=None):
         """
-        Scales the SI following Surf. Interface Anal. 2004; 36: 203–212 to
-        "normalize" the poissonian data for decomposition analysis
+        Scales the SI following Surf. Interface Anal. 2004; 36: 203–212 
+        to "normalize" the poissonian data for decomposition analysis
 
         Parameters
         ----------
@@ -704,15 +796,18 @@ class MVA():
         signal_mask  : boolen numpy array
         """
         messages.information(
-            "Scaling the data to normalize the (presumably) Poissonian noise")
+            "Scaling the data to normalize the (presumably)"
+            " Poissonian noise")
         refold = self.unfold_if_multidim()
         dc = self.data
         if navigation_mask is None:
             navigation_mask = slice(None)
         else:
-            navigation_mask = navigation_mask.ravel()
+            navigation_mask = ~navigation_mask.ravel()
         if signal_mask is None:
             signal_mask = slice(None)
+        else:
+            signal_mask = ~signal_mask
         # Rescale the data to gaussianize the poissonian noise
         aG = dc[:,signal_mask][navigation_mask,:].sum(1).squeeze()
         bH = dc[:,signal_mask][navigation_mask,:].sum(0).squeeze()
@@ -720,39 +815,25 @@ class MVA():
         if (aG < 0).any() or (bH < 0).any():
             messages.warning_exit(
             "Data error: negative values\n"
-            "Are you sure that the data follow a poissonian distribution?")
-        # Update the spatial and energy masks so it does not include rows
-        # or colums that sum zero.
-        aG0 = (aG == 0)
-        bH0 = (bH == 0)
-        if aG0.any():
-            if isinstance(navigation_mask, slice):
-                # Convert the slice into a mask before setting its values
-                navigation_mask = np.ones((self.data.shape[0]),dtype = 'bool')
-            # Set colums summing zero as masked
-            navigation_mask[aG0] = False
-            aG = aG[aG0 == False]
-        if bH0.any():
-            if isinstance(signal_mask, slice):
-                # Convert the slice into a mask before setting its values
-                signal_mask = np.ones((self.data.shape[1]), dtype = 'bool')
-            # Set rows summing zero as masked
-            signal_mask[bH0] = False
-            bH = bH[bH0 == False]
+            "Are you sure that the data follow a poissonian "
+            "distribution?")
+
         self._root_aG = np.sqrt(aG)[:, np.newaxis]
         self._root_bH = np.sqrt(bH)[np.newaxis, :]
+        # We first disable numpy's warning when the result of an
+        # operation produces nans
+        np.seterr(invalid='ignore')
+        dc[:,signal_mask][navigation_mask,:] /= (self._root_aG * 
+            self._root_bH)
+        # Enable numpy warning
+        np.seterr(invalid=None)
+        #Set the nans resulting from 0/0 to zero
         dc[:,signal_mask][navigation_mask,:] = \
-            (dc[:,signal_mask][navigation_mask,:] /
-                (self._root_aG * self._root_bH))
+            np.nan_to_num(dc[:,signal_mask][navigation_mask,:])
+        
         if refold is True:
             print "Automatically refolding the SI after scaling"
             self.fold()
-        if return_masks is True:
-            if isinstance(navigation_mask, slice):
-                navigation_mask = None
-            if isinstance(signal_mask, slice):
-                signal_mask = None
-            return navigation_mask, signal_mask
 
     def undo_treatments(self):
         """Undo normalize_poissonian_noise"""
@@ -792,15 +873,15 @@ class LearningResults(object):
         """
         kwargs = {}
         for attribute in [
-            v for v in dir(self) if type(getattr(self,v)) != types.MethodType
-                and not v.startswith('_')]:
+            v for v in dir(self) if type(getattr(self,v)) != 
+            types.MethodType and not v.startswith('_')]:
             kwargs[attribute] = self.__getattribute__(attribute)
         np.savez(filename, **kwargs)
 
 
     def load(self, filename):
-        """Load the results of a previous decomposition and demixing analysis
-        from a file.
+        """Load the results of a previous decomposition and
+         demixing analysis from a file.
 
         Parameters
         ----------
@@ -852,12 +933,12 @@ class LearningResults(object):
             del self.pc
             
         if hasattr(self, 'ica_scores'):
-            self.bss_loadings = self.ica_loadings
-            del self.self.ica_loadings
+            self.bss_loadings = self.ica_scores
+            del self.ica_scores
             
         if hasattr(self, 'ica_factors'):
             self.bss_factors = self.ica_factors
-            del self.self.ica_factors
+            del self.ica_factors
 
         #######################################################
         
@@ -868,8 +949,8 @@ class LearningResults(object):
         self.summary()
 
     def summary(self):
-        """Prints a summary of the decomposition and demixing parameters to the 
-        stdout
+        """Prints a summary of the decomposition and demixing parameters
+         to the stdout
         """
         print
         print "Decomposition parameters:"
@@ -901,6 +982,7 @@ class LearningResults(object):
         self.factors = self.factors[:,:n]
         
     def _transpose_results(self):
-        self.factors, self.loadings, self.bss_factors, self.bss_loadings = \
-        self.loadings, self.factors, self.bss_loadings, self.bss_factors
+        (self.factors, self.loadings, self.bss_factors, 
+            self.bss_loadings) = (self.loadings, self.factors, 
+                self.bss_loadings, self.bss_factors)
         
