@@ -21,10 +21,13 @@ import os
 import tempfile
 
 import numpy as np
+import numpy.linalg
+
 import traits.api as t
+import scipy.odr as odr
+from scipy.optimize import (leastsq,fmin, fmin_cg, fmin_ncg, fmin_bfgs,
+    fmin_cobyla, fmin_l_bfgs_b, fmin_tnc, fmin_powell)
 
-from hyperspy.estimators import Estimators
-from hyperspy.optimizers import Optimizers
 from hyperspy import messages
 import hyperspy.drawing.spectrum
 from hyperspy.drawing.utils import on_figure_window_close
@@ -34,8 +37,9 @@ from hyperspy.defaults_parser import preferences
 from hyperspy.axes import generate_axis
 from hyperspy.exceptions import WrongObjectError
 from hyperspy.decorators import interactive_range_selector
+from hyperspy.misc.mpfit.mpfit import mpfit
 
-class Model(list, Optimizers, Estimators):
+class Model(list):
     """Build and fit a model
     
     Parameters
@@ -50,7 +54,7 @@ class Model(list, Optimizers, Estimators):
         self.auto_update_plot = False
         self.spectrum = spectrum
         self.axes_manager = self.spectrum.axes_manager
-        self.axis = self.axes_manager._slicing_axes[0]
+        self.axis = self.axes_manager.signal_axes[0]
         self.axes_manager.connect(self.charge)
          
         self.free_parameters_boundaries = None
@@ -112,7 +116,7 @@ class Model(list, Optimizers, Estimators):
         list.__delitem__(self,object)
         self._touch()
     
-    def remove(self, object, touch = True):
+    def remove(self, object, touch=True):
         list.remove(self,object)
         if touch is True:
             self._touch() 
@@ -133,7 +137,7 @@ class Model(list, Optimizers, Estimators):
         scale to obtain the correct axis and origin after convolution with the
         lowloss spectrum.
         """
-        ll_axis = self.low_loss.axes_manager._slicing_axes[0]
+        ll_axis = self.low_loss.axes_manager.signal_axes[0]
         dimension = self.axis.size + ll_axis.size - 1
         step = self.axis.scale
         knot_position = ll_axis.size - ll_axis.value2index(0) - 1
@@ -142,6 +146,7 @@ class Model(list, Optimizers, Estimators):
                 
     def connect_parameters2update_plot(self):   
         for component in self:
+            component.connect(self.update_plot)
             for parameter in component.parameters:
                 if self.spectrum._plot is not None:
                     parameter.connect(self.update_plot)
@@ -149,6 +154,7 @@ class Model(list, Optimizers, Estimators):
     
     def disconnect_parameters2update_plot(self):
         for component in self:
+            component.disconnect(self.update_plot)
             for parameter in component.parameters:
                 parameter.disconnect(self.update_plot)
                 parameter.connection_active = False
@@ -160,7 +166,7 @@ class Model(list, Optimizers, Estimators):
                 parameter.connection_active = tof
         self.auto_update_plot = tof
 
-    def generate_data_from_model(self, out_of_range_to_nan = True):
+    def generate_data_from_model(self, out_of_range_to_nan=True):
         """Generate a SI with the current model
         
         The SI is stored in self.model_cube
@@ -238,13 +244,13 @@ class Model(list, Optimizers, Estimators):
             if component.active:
                 for param in component.free_parameters:
                     if param._number_of_elements == 1:
-                        limited = [0,0]
+                        limited = [False,False]
                         limits = [0,0]
                         if param.bmin is not None:
-                            limited[0] = 1
+                            limited[0] = True
                             limits[0] = param.bmin
                         if param.bmax is not None:
-                            limited[1] = 1
+                            limited[1] = True
                             limits[1] = param.bmax
                         self.mpfit_parinfo.append(
                         {'limited' : limited,
@@ -260,7 +266,7 @@ class Model(list, Optimizers, Estimators):
             component.store_current_parameters_in_map(
             tuple(self.axes_manager._indexes))
 
-    def charge(self, only_fixed = False):
+    def charge(self, only_fixed=False):
         """Charge the parameters for the current spectrum from the parameters 
         array
         
@@ -283,7 +289,7 @@ class Model(list, Optimizers, Estimators):
     def update_plot(self):
         if self.spectrum._plot is not None:
             try:
-                for line in self.spectrum._plot.spectrum_plot.ax_lines:
+                for line in self.spectrum._plot.signal_plot.ax_lines:
                         line.update()
             except:
                 self.disconnect_parameters2update_plot()
@@ -309,7 +315,7 @@ class Model(list, Optimizers, Estimators):
                 counter += component._nfree_param
 
     # Defines the functions for the fitting process -------------------------
-    def _model2plot(self, axes_manager, out_of_range2nans = True):
+    def _model2plot(self, axes_manager, out_of_range2nans=True):
         old_axes_manager = None
         if axes_manager is not self.axes_manager:
             old_axes_manager = self.axes_manager
@@ -326,7 +332,7 @@ class Model(list, Optimizers, Estimators):
             s = ns
         return s
     
-    def __call__(self, non_convolved = False, onlyactive = False) :
+    def __call__(self, non_convolved=False, onlyactive=False) :
         """Returns the corresponding model for the current coordinates
         
         Parameters
@@ -389,7 +395,7 @@ class Model(list, Optimizers, Estimators):
             return to_return
 
     # TODO: the way it uses the axes
-    def set_data_range_in_pixels(self, i1 = None, i2 = None):
+    def _set_signal_range_in_pixels(self, i1=None, i2=None):
         """Use only the selected spectral range in the fitting routine.
         
         Parameters
@@ -409,7 +415,7 @@ class Model(list, Optimizers, Estimators):
             self.update_plot()
             
     @interactive_range_selector   
-    def set_data_range_in_units(self, x1 = None, x2 = None):
+    def set_signal_range(self, x1=None, x2=None):
         """Use only the selected spectral range defined in its own units in the 
         fitting routine.
         
@@ -423,11 +429,11 @@ class Model(list, Optimizers, Estimators):
         To use the full energy range call the function without arguments.
         """
         i1, i2 = self.axis.value2index(x1), self.axis.value2index(x2)
-        self.set_data_range_in_pixels(i1, i2)
+        self._set_signal_range_in_pixels(i1, i2)
 
-    def remove_data_range_in_pixels(self, i1 = None, i2= None):
-        """Removes the data in the given range from the data range that will be 
-        used by the fitting rountine
+    def _remove_signal_range_in_pixels(self, i1=None, i2=None):
+        """Removes the data in the given range from the data range that 
+        will be used by the fitting rountine
         
         Parameters
         ----------
@@ -439,9 +445,9 @@ class Model(list, Optimizers, Estimators):
             self.update_plot()
 
     @interactive_range_selector    
-    def remove_data_range_in_units(self, x1 = None, x2= None):
-        """Removes the data in the given range from the data range that will be 
-        used by the fitting rountine
+    def remove_signal_range(self, x1=None, x2=None):
+        """Removes the data in the given range from the data range that 
+        will be used by the fitting rountine
         
         Parameters
         ----------
@@ -450,15 +456,15 @@ class Model(list, Optimizers, Estimators):
         
         """
         i1, i2 = self.axis.value2index(x1), self.axis.value2index(x2)
-        self.remove_data_range_in_pixels(i1, i2)
+        self._remove_signal_range_in_pixels(i1, i2)
         
-    def reset_data_range(self):
+    def reset_signal_range(self):
         '''Resets the data range'''
-        self.set_data_range_in_pixels()
+        self._set_signal_range_in_pixels()
     
-    def add_data_range_in_pixels(self, i1 = None, i2= None):
-        """Adds the data in the given range from the data range that will be 
-        used by the fitting rountine
+    def _add_signal_range_in_pixels(self, i1=None, i2=None):
+        """Adds the data in the given range from the data range that 
+        will be used by the fitting rountine
         
         Parameters
         ----------
@@ -470,9 +476,9 @@ class Model(list, Optimizers, Estimators):
             self.update_plot()
 
     @interactive_range_selector    
-    def add_data_range_in_units(self, x1 = None, x2= None):
-        """Adds the data in the given range from the data range that will be 
-        used by the fitting rountine
+    def add_signal_range(self, x1=None, x2=None):
+        """Adds the data in the given range from the data range that 
+        will be used by the fitting rountine
         
         Parameters
         ----------
@@ -481,9 +487,9 @@ class Model(list, Optimizers, Estimators):
         
         """
         i1, i2 = self.axis.value2index(x1), self.axis.value2index(x2)
-        self.add_data_range_in_pixels(i1, i2)
+        self._add_signal_range_in_pixels(i1, i2)
         
-    def reset_the_data_range(self):
+    def reset_the_signal_range(self):
         self.channel_switches[:] = True
         if self.auto_update_plot is True:
             self.update_plot()
@@ -587,14 +593,299 @@ class Model(list, Optimizers, Estimators):
     
     def _jacobian4odr(self,param,x):
         return self._jacobian(param, x)
-                
-    def multifit(self, mask = None, fitter = None, 
-                 charge_only_fixed = False, grad = False, autosave = False, 
-                 autosave_every = 10, bounded = False, **kwargs):
         
+    def calculate_p_std(self, p0, method, *args):
+        print "Estimating the standard deviation"
+        f = self._poisson_likelihood_function if method == 'ml' \
+        else self._errfunc2
+        hess = approx_hessian(p0,f,*args)
+        ihess = np.linalg.inv(hess)
+        p_std = np.sqrt(1./np.diag(ihess))
+        return p_std
+
+    def _poisson_likelihood_function(self,param,y, weights=None):
+        """Returns the likelihood function of the model for the given
+        data and parameters
+        """
+        mf = self._model_function(param)
+        return -(y*np.log(mf) - mf).sum()
+
+    def _gradient_ml(self,param, y, weights=None):
+        mf = self._model_function(param)
+        return -(self._jacobian(param, y)*(y/mf - 1)).sum(1)
+
+
+    def _errfunc(self,param, y, weights=None):
+        errfunc = self._model_function(param) - y
+        if weights is None:
+            return errfunc
+        else:
+            return errfunc * weights
+    def _errfunc2(self,param, y, weights=None):
+        if weights is None:
+            return ((self._errfunc(param, y))**2).sum()
+        else:
+            return ((weights * self._errfunc(param, y))**2).sum()
+
+    def _gradient_ls(self,param, y, weights=None):
+        gls =(2*self._errfunc(param, y, weights) * 
+        self._jacobian(param, y)).sum(1)
+        return gls
+        
+    def _errfunc4mpfit(self, p, fjac=None, x=None, y=None,
+        weights = None):
+        if fjac is None:
+            errfunc = self._model_function(p) - y
+            if weights is not None:
+                errfunc *= weights
+            jacobian = None
+            status = 0
+            return [status, errfunc]
+        else:
+            return [0, self._jacobian(p,y).T]
+        
+    def fit(self, fitter=None, method='ls', grad=False, weights=None,
+            bounded=False, ext_bounding=False, update_plot=False, 
+            **kwargs):
+        """Fits the model to the experimental data
+        
+        Parameters
+        ----------
+        fitter : {None, "leastsq", "odr", "mpfit", "fmin"}
+            The optimizer to perform the fitting. If None the fitter
+            defined in the Preferences is used. leastsq is the most 
+            stable but it does not support bounding. mpfit supports
+            bounding. fmin is the only one that supports 
+            maximum likelihood estimation, but it is less robust than 
+            the Levenberg–Marquardt based leastsq and mpfit, and it is 
+            better to use it after one of them to refine the estimation.
+        method : {'ls', 'ml'}
+            Choose 'ls' (default) for least squares and 'ml' for 
+            maximum-likelihood estimation. The latter only works with 
+            fitter = 'fmin'.
+        grad : bool
+            If True, the analytical gradient is used if defined to 
+            speed up the estimation. 
+        weights : {None, True, numpy.array}
+            If None, performs standard least squares. If True 
+            performs weighted least squares where the weights are 
+            calculated using spectrum.Spectrum.estimate_variance. 
+            Alternatively, external weights can be supplied by passing
+            a weights array of the same dimensions as the signal.
+        ext_bounding : bool
+            If True, enforce bounding by keeping the value of the 
+            parameters constant out of the defined bounding area.
+        bounded : bool
+            If True performs bounded optimization if the fitter 
+            supports it. Currently only mpfit support bounding. 
+        update_plot : bool
+            If True, the plot is updated during the optimization 
+            process. It slows down the optimization but it permits
+            to visualize the optimization evolution. 
+        
+        **kwargs : key word arguments
+            Any extra key word argument will be passed to the chosen
+            fitter
+            
+        See Also
+        --------
+        multifit
+            
+        """
         if fitter is None:
             fitter = preferences.Model.default_fitter
-            print('Fitter: %s' % fitter) 
+        switch_aap = (update_plot != self.auto_update_plot)
+        if switch_aap is True:
+            self.set_auto_update_plot(update_plot)
+        self.p_std = None
+        self._set_p0()
+        if ext_bounding:
+            self._enable_ext_bounding()
+        if grad is False :
+            approx_grad = True
+            jacobian = None
+            odr_jacobian = None
+            grad_ml = None
+            grad_ls = None
+        else :
+            approx_grad = False
+            jacobian = self._jacobian
+            odr_jacobian = self._jacobian4odr
+            grad_ml = self._gradient_ml
+            grad_ls = self._gradient_ls
+        if method == 'ml':
+            weights = None
+        if weights is True:
+            if self.spectrum.variance is None:
+                self.spectrum.estimate_variance()
+            weights = 1. / np.sqrt(self.spectrum.variance.__getitem__(
+            self.axes_manager._getitem_tuple)[self.channel_switches])
+        elif weights is not None:
+            weights = weights.__getitem__(
+                self.axes_manager._getitem_tuple)[
+                    self.channel_switches]
+        args = (self.spectrum()[self.channel_switches], 
+        weights)
+        
+        # Least squares "dedicated" fitters
+        if fitter == "leastsq":
+            output = \
+            leastsq(self._errfunc, self.p0[:], Dfun = jacobian,
+            col_deriv=1, args = args, full_output = True, **kwargs)
+            
+            self.p0 = output[0]
+            var_matrix = output[1]
+            # In Scipy 0.7 sometimes the variance matrix is None (maybe a 
+            # bug?) so...
+            if var_matrix is not None:
+                self.p_std = np.sqrt(np.diag(var_matrix))
+            self.fit_output = output
+        
+        elif fitter == "odr":
+            modelo = odr.Model(fcn = self._function4odr, 
+            fjacb = odr_jacobian)
+            mydata = odr.RealData(self.axis.axis[self.channel_switches],
+            self.spectrum()[self.channel_switches],
+            sx = None,
+            sy = (1/weights if weights is not None else None))
+            myodr = odr.ODR(mydata, modelo, beta0=self.p0[:])
+            myoutput = myodr.run()
+            result = myoutput.beta
+            self.p_std = myoutput.sd_beta
+            self.p0 = result
+            self.fit_output = myoutput
+            
+        elif fitter == 'mpfit':
+            autoderivative = 1
+            if grad is True:
+                autoderivative = 0
+
+            if bounded is True:
+                self.set_mpfit_parameters_info()
+            elif bounded is False:
+                self.mpfit_parinfo = None
+            m = mpfit(self._errfunc4mpfit, self.p0[:], 
+                parinfo=self.mpfit_parinfo, functkw= {
+                'y': self.spectrum()[self.channel_switches], 
+                'weights' :weights}, autoderivative = autoderivative,
+                quiet = 1)
+            self.p0 = m.params
+            self.p_std = m.perror
+            self.fit_output = m
+            
+        else:          
+        # General optimizers (incluiding constrained ones(tnc,l_bfgs_b)
+        # Least squares or maximum likelihood
+            if method == 'ml':
+                tominimize = self._poisson_likelihood_function
+                fprime = grad_ml
+            elif method == 'ls':
+                tominimize = self._errfunc2
+                fprime = grad_ls
+                        
+            # OPTIMIZERS
+            
+            # Simple (don't use gradient)
+            if fitter == "fmin" :
+                self.p0 = fmin(
+                    tominimize, self.p0, args = args, **kwargs)
+            elif fitter == "powell" :
+                self.p0 = fmin_powell(tominimize, self.p0, args = args, 
+                **kwargs)
+            
+            # Make use of the gradient
+            elif fitter == "cg" :
+                self.p0 = fmin_cg(tominimize, self.p0, fprime = fprime,
+                args= args, **kwargs)
+            elif fitter == "ncg" :
+                self.p0 = fmin_ncg(tominimize, self.p0, fprime = fprime,
+                args = args, **kwargs)
+            elif fitter == "bfgs" :
+                self.p0 = fmin_bfgs(
+                    tominimize, self.p0, fprime = fprime,
+                    args = args, **kwargs)
+            
+            # Constrainded optimizers
+            
+            # Use gradient
+            elif fitter == "tnc":
+                if bounded is True:
+                    self.set_boundaries()
+                elif bounded is False:
+                    self.self.free_parameters_boundaries = None
+                self.p0 = fmin_tnc(tominimize, self.p0, fprime = fprime,
+                args = args, bounds = self.free_parameters_boundaries, 
+                approx_grad = approx_grad, **kwargs)[0]
+            elif fitter == "l_bfgs_b":
+                if bounded is True:
+                    self.set_boundaries()
+                elif bounded is False:
+                    self.self.free_parameters_boundaries = None
+                self.p0 = fmin_l_bfgs_b(tominimize, self.p0,
+                    fprime=fprime, args=args, 
+                    bounds=self.free_parameters_boundaries, 
+                    approx_grad = approx_grad, **kwargs)[0]
+            else:
+                print \
+                """
+                The %s optimizer is not available.
+
+                Available optimizers:
+                Unconstrained:
+                --------------
+                Only least Squares: leastsq and odr
+                General: fmin, powell, cg, ncg, bfgs
+
+                Cosntrained:
+                ------------
+                tnc and l_bfgs_b
+                """ % fitter
+                
+        
+        if np.iterable(self.p0) == 0:
+            self.p0 = (self.p0,)
+        self._charge_p0(p_std = self.p_std)
+        self.set()
+        if ext_bounding is True:
+            self._disable_ext_bounding()
+        if switch_aap is True:
+            self.set_auto_update_plot(~update_plot)
+        if (update_plot is False and self.spectrum._plot 
+                is not None):
+            self.update_plot()
+                
+    def multifit(self, mask=None, charge_only_fixed=False,
+                 autosave=False, autosave_every=10, **kwargs):
+        """Fit the data to the model at all the positions of the 
+        navigation dimensions.        
+        
+        Parameters
+        ----------
+        
+        mask : {None, numpy.array}
+            To mask (do not fit) at certain position pass a numpy.array
+            of type bool where True indicates that the data will not be
+            fitted at the given position.
+        charge_only_fixed : bool
+            If True, only the fixed parameters values will be updated
+            when changing the positon.
+        autosave : bool
+            If True, the result of the fit will be saved automatically
+            with a frequency defined by autosave_every.
+        autosave_every : int
+            Save the result of fitting every given number of spectra.
+        
+        **kwargs : key word arguments
+            Any extra key word argument will be passed to 
+            the fit method. See the fit method documentation for 
+            a list of valid arguments.
+            
+        See Also
+        --------
+        fit
+            
+        """
+        
         if autosave is not False:
             fd, autosave_fn = tempfile.mkstemp(prefix = 'hyperspy_autosave-', 
             dir = '.', suffix = '.npz')
@@ -608,32 +899,33 @@ class Model(list, Optimizers, Estimators):
         if mask is not None and \
         (mask.shape != tuple(self.axes_manager.navigation_shape)):
            messages.warning_exit(
-           "The mask must be an array with the same espatial dimensions as the" 
-           "navigation shape, %s" % self.axes_manager.navigation_shape)
+           "The mask must be a numpy array of boolen type with "
+           " the same shape as the navigation: %s" % 
+           self.axes_manager.navigation_shape)
         masked_elements = 0 if mask is None else mask.sum()
         pbar = progressbar.progressbar(
         maxval = (np.cumprod(self.axes_manager.navigation_shape)[-1] - 
         masked_elements))
-        if bounded is True:
+        if 'bounded' in kwargs and kwargs['bounded'] is True:
             if fitter == 'mpfit':
                 self.set_mpfit_parameters_info()
-                bounded = None
+                kwargs['bounded'] = None
             elif fitter in ("tnc", "l_bfgs_b"):
                 self.set_boundaries()
-                bounded = None
+                kwargs['bounded'] = None
             else:
                 messages.information(
                 "The chosen fitter does not suppport bounding."
-                "If you require boundinig please select one of the following"
-                "fitters instead: mpfit, tnc, l_bfgs_b")
-                bounded = False
+                "If you require boundinig please select one of the "
+                "following fitters instead: mpfit, tnc, l_bfgs_b")
+                kwargs['bounded'] = False
         i = 0
-        for index in np.ndindex(tuple(self.axes_manager.navigation_shape)):
+        for index in np.ndindex(tuple(
+                self.axes_manager.navigation_shape)):
             if mask is None or not mask[index]:
                 self.axes_manager.set_not_slicing_indexes(index)
                 self.charge(only_fixed = charge_only_fixed)
-                self.fit(fitter = fitter, grad = grad, bounded = bounded, 
-                         **kwargs)
+                self.fit(**kwargs)
                 i += 1
                 pbar.update(i)
             if autosave is True and i % autosave_every  == 0:
@@ -641,12 +933,19 @@ class Model(list, Optimizers, Estimators):
         pbar.finish()
         if autosave is True:
             messages.information(
-            'Deleting the temporary file %s pixels' % (autosave_fn + 'npz'))
+            'Deleting the temporary file %s pixels' % (
+                autosave_fn + 'npz'))
             os.remove(autosave_fn + '.npz')
 
             
-    def save_parameters2file(self,filename):
-        """Save the parameters array in binary format"""
+    def save_parameters2file(self, filename):
+        """Save the parameters array in binary format
+        
+        Parameters
+        ----------
+        filename : str
+        
+        """
         kwds = {}
         i = 0
         for component in self:
@@ -658,8 +957,14 @@ class Model(list, Optimizers, Estimators):
         np.savez(filename, **kwds)
 
     def load_parameters_from_file(self,filename):
-        """Loads the parameters array from  a binary file written with the
-        'save_parameters2file' function"""
+        """Loads the parameters array from  a binary file written with 
+        the 'save_parameters2file' function
+        
+        Paramters
+        ---------
+        filename : str
+        
+        """
         
         f = np.load(filename)
         i = 0
@@ -673,14 +978,15 @@ class Model(list, Optimizers, Estimators):
         self.charge()
            
     def plot(self, auto_update_plot = True):
-        """Plots the current spectrum to the screen and a map with a cursor to 
-        explore the SI.
+        """Plots the current spectrum to the screen and a map with a 
+        cursor to explore the SI.
+        
         """
         
         # If new coordinates are assigned
         self.spectrum.plot()
         _plot = self.spectrum._plot
-        l1 = _plot.spectrum_plot.ax_lines[0]
+        l1 = _plot.signal_plot.ax_lines[0]
         color = l1.line.get_color()
         l1.line_properties_helper(color, 'scatter')
         l1.set_properties()
@@ -690,16 +996,16 @@ class Model(list, Optimizers, Estimators):
         l2.line_properties_helper('blue', 'line')        
         # Add the line to the figure
           
-        _plot.spectrum_plot.add_line(l2)
+        _plot.signal_plot.add_line(l2)
         l2.plot()
         self.connect_parameters2update_plot()
-        on_figure_window_close(_plot.spectrum_plot.figure, 
-                                      self.disconnect_parameters2update_plot)
+        on_figure_window_close(_plot.signal_plot.figure, 
+                                self.disconnect_parameters2update_plot)
         self.set_auto_update_plot(True)
         self._plot = self.spectrum._plot
         # TODO Set autoupdate to False on close
         
-    def set_current_values_to(self, components_list = None, mask = None):
+    def set_current_values_to(self, components_list=None, mask=None):
         if components_list is None:
             components_list = []
             for comp in self:
@@ -727,7 +1033,7 @@ class Model(list, Optimizers, Estimators):
                 parameter.ext_bounded = False
                 
     def export_results(self, folder=None, format=None, save_std=False,
-                       only_free=True, only_active = True):
+                       only_free=True, only_active=True):
         """Export the results of the parameters of the model to the desired
         folder.
         
@@ -795,10 +1101,11 @@ class Model(list, Optimizers, Estimators):
         print "Components\tParameter\tValue"
         for component in self:
             if component.active is True:
-                print(component.name )
+                if component.name:
+                    print(component.name)
+                else:
+                    print(component._id_name)
                 parameters = component.free_parameters if only_free \
                     else component.parameter
                 for parameter in parameters:
                     print("\t\t%s\t%f" % (parameter.name, parameter.value))
-
-        
