@@ -184,21 +184,21 @@ class Signal(t.HasTraits, MVA):
         return self.data.__getitem__(axes_manager._getitem_tuple)
 
     def _get_hse_1D_explorer(self, *args, **kwargs):
-        islice = self.axes_manager._slicing_axes[0].index_in_array
-        inslice = self.axes_manager._non_slicing_axes[0].index_in_array
+        islice = self.axes_manager.signal_axes[0].index_in_array
+        inslice = self.axes_manager.navigation_axes[0].index_in_array
         if islice > inslice:
             return self.data.squeeze()
         else:
             return self.data.squeeze().T
 
     def _get_hse_2D_explorer(self, *args, **kwargs):
-        islice = self.axes_manager._slicing_axes[0].index_in_array
+        islice = self.axes_manager.signal_axes[0].index_in_array
         data = np.nan_to_num(self.data).sum(islice)
         return data
 
     def _get_hie_explorer(self, *args, **kwargs):
-        isslice = [self.axes_manager._slicing_axes[0].index_in_array,
-                   self.axes_manager._slicing_axes[1].index_in_array]
+        isslice = [self.axes_manager.signal_axes[0].index_in_array,
+                   self.axes_manager.signal_axes[1].index_in_array]
         isslice.sort()
         data = self.data.sum(isslice[1]).sum(isslice[0])
         return data
@@ -234,39 +234,22 @@ class Signal(t.HasTraits, MVA):
 
         if axes_manager.signal_dimension == 1:
             # Hyperspectrum
-
             self._plot = mpl_hse.MPL_HyperSpectrum_Explorer()
-            self._plot.spectrum_data_function = self.__call__
-            self._plot.spectrum_title = self.mapped_parameters.title
-            self._plot.xlabel = '%s (%s)' % (
-                self.axes_manager._slicing_axes[0].name,
-                self.axes_manager._slicing_axes[0].units)
-            self._plot.ylabel = 'Intensity'
-            self._plot.axes_manager = axes_manager
-            self._plot.axis = self.axes_manager._slicing_axes[0].axis
-
-            # Image properties
-            if self.axes_manager._non_slicing_axes:
-                self._plot.image_data_function = self._get_explorer
-                self._plot.image_title = ''
-                if self.axes_manager.navigation_dimension == 1:
-                    scalebar_axis = self.axes_manager._slicing_axes[0]
-                else:
-                    scalebar_axis = self.axes_manager._non_slicing_axes[-1]
-                self._plot.pixel_size = scalebar_axis.scale
-                self._plot.pixel_units = scalebar_axis.units
-            self._plot.plot()
             
         elif axes_manager.signal_dimension == 2:
             self._plot = mpl_hie.MPL_HyperImage_Explorer()
-            self._plot.image_data_function = self.__call__
-            self._plot.navigator_data_function = self._get_explorer
-            self._plot.axes_manager = axes_manager
-            self._plot.image_title = self.mapped_parameters.title
-            self._plot.plot()
-
         else:
-            messages.warning_exit('Plotting is not supported for this view')
+            raise ValueError('Plotting is not supported for this view')
+        
+        self._plot.axes_manager = axes_manager
+        self._plot.signal_data_function = self.__call__
+        self._plot.signal_title = self.mapped_parameters.title
+
+        # Navigator properties
+        if self.axes_manager.navigation_axes:
+            self._plot.navigator_data_function = self._get_explorer
+        self._plot.plot()
+            
 
     def plot_residual(self, axes_manager=None):
         """Plot the residual between original data and reconstructed data
@@ -281,7 +264,7 @@ class Signal(t.HasTraits, MVA):
             print "Object does not have any residual information.  Is it a \
 reconstruction created using either get_decomposition_model or get_bss_model methods?"
 
-    def save(self, filename, **kwds):
+    def save(self, filename=None, overwrite=None, **kwds):
         """Saves the signal in the specified format.
 
         The function gets the format from the extension. You can use:
@@ -300,9 +283,26 @@ reconstruction created using either get_decomposition_model or get_bss_model met
 
         Parameters
         ----------
-        filename : str
+        filename : str or None
+            If None and mapped_parameters.filename or 
+            mapped_parameters.original_filename are defined, the
+            filename and extension will be taken from them in this 
+            order.
+        overwrite : None, bool
+            If None, if the file exists it will query the user. If 
+            True(False) it (does not) overwrites the file if it exists.
         """
-        io.save(filename, self, **kwds)
+        if filename is None:
+            if self.mapped_parameters.has_item('filename'):
+                filename = self.mapped_parameters.filename
+            elif self.mapped_parameters.has_item('original_filename'):
+                filename = self.mapped_parameters.original_filename
+                self.mapped_parameters.filename = \
+                    self.mapped_parameters.original_filename
+            else:
+                raise ValueError('File name not defined')
+
+        io.save(filename, self, overwrite=overwrite, **kwds)
 
     def _replot(self):
         if self._plot is not None:
@@ -321,17 +321,21 @@ reconstruction created using either get_decomposition_model or get_bss_model met
             print("%s size: %i" %
             (axis.name, dc.shape[axis.index_in_array]))
 
-    def crop_in_pixels(self, axis, i1 = None, i2 = None):
+    def crop_in_pixels(self, axis, i1=None, i2=None, copy=True):
         """Crops the data in a given axis. The range is given in pixels
         axis : int
         i1 : int
             Start index
         i2 : int
             End index
+        copy : bool
+            If True makes a copy of the data, otherwise the cropping
+            performs just a view.
 
         See also:
         ---------
         crop_in_units
+        
         """
         axis = self._get_positive_axis_index_index(axis)
         if i1 is not None:
@@ -345,15 +349,18 @@ reconstruction created using either get_decomposition_model or get_bss_model met
         self.get_dimensions_from_data()
         self.squeeze()
 
-    def crop_in_units(self, axis, x1 = None, x2 = None):
+    def crop_in_units(self, axis, x1=None, x2=None, copy=True):
         """Crops the data in a given axis. The range is given in the units of
         the axis
 
         axis : int
-        i1 : int
-            Start index
-        i2 : int
-            End index
+        i1 : float
+            Start value
+        i2 : float
+            End value
+        copy : bool
+            If True makes a copy of the data, otherwise the cropping
+            performs just a view.
 
         See also:
         ---------
@@ -362,7 +369,7 @@ reconstruction created using either get_decomposition_model or get_bss_model met
         """
         i1 = self.axes_manager.axes[axis].value2index(x1)
         i2 = self.axes_manager.axes[axis].value2index(x2)
-        self.crop_in_pixels(axis, i1, i2)
+        self.crop_in_pixels(axis, i1, i2,copy=copy)
 
     @auto_replot
     def roll_xy(self, n_x, n_y = 1):
@@ -454,11 +461,16 @@ reconstruction created using either get_decomposition_model or get_bss_model met
             cut_node = range(0, rounded+step, step)
         else:
             cut_node = np.array([0] + steps).cumsum()
+        signal_dict = self._get_signal_dict()
+        axes_dict = signal_dict['axes']
         for i in xrange(len(cut_node)-1):
+            axes_dict[axis]['offset'] = \
+                self.axes_manager.axes[axis].index2value(cut_node[i])
             data = self.data[
             (slice(None), ) * axis + (slice(cut_node[i], cut_node[i + 1]),
             Ellipsis)]
-            s = Signal({'data': data})
+            signal_dict['data'] = data
+            s = Signal(signal_dict)
             # TODO: When copying plotting does not work
 #            s.axes = copy.deepcopy(self.axes_manager)
             s.get_dimensions_from_data()
@@ -554,8 +566,8 @@ reconstruction created using either get_decomposition_model or get_bss_model met
             return False
         steady_axes = [
                         axis.index_in_array for axis in
-                        self.axes_manager._slicing_axes]
-        unfolded_axis = self.axes_manager._non_slicing_axes[-1].index_in_array
+                        self.axes_manager.signal_axes]
+        unfolded_axis = self.axes_manager.navigation_axes[-1].index_in_array
         self._unfold(steady_axes, unfolded_axis)
 
     def unfold_signal_space(self):
@@ -566,8 +578,8 @@ reconstruction created using either get_decomposition_model or get_bss_model met
             return False
         steady_axes = [
                         axis.index_in_array for axis in
-                        self.axes_manager._non_slicing_axes]
-        unfolded_axis = self.axes_manager._slicing_axes[-1].index_in_array
+                        self.axes_manager.navigation_axes]
+        unfolded_axis = self.axes_manager.signal_axes[-1].index_in_array
         self._unfold(steady_axes, unfolded_axis)
 
     @auto_replot
@@ -584,10 +596,11 @@ reconstruction created using either get_decomposition_model or get_bss_model met
             axis = len(self.data.shape) + axis
         return axis
 
-    def iterate_axis(self, axis = -1):
-        # We make a copy to guarantee that the data in contiguous, otherwise
-        # it will not return a view of the data
-        self.data = self.data.copy()
+    def iterate_axis(self, axis = -1, copy=True):
+        # We make a copy to guarantee that the data in contiguous,
+        # otherwise it will not return a view of the data
+        if copy is True:
+            self.data = self.data.copy()
         axis = self._get_positive_axis_index_index(axis)
         unfolded_axis = axis - 1
         new_shape = [1] * len(self.data.shape)
@@ -600,9 +613,30 @@ reconstruction created using either get_decomposition_model or get_bss_model met
             getitem[axis] = slice(None)
             getitem[unfolded_axis] = i
             yield(data[getitem])
+            
+    def _iterate_signal(self, copy=True):
+        # We make a copy to guarantee that the data in contiguous,
+        # otherwise it will not return a view of the data
+        if copy is True:
+            self.data = self.data.copy()
+        axes = [axis.index_in_array for 
+                axis in self.axes_manager.signal_axes]
+        unfolded_axis = self.axes_manager.navigation_axes[-1].index_in_array
+        new_shape = [1] * len(self.data.shape)
+        for axis in axes:
+            new_shape[axis] = self.data.shape[axis]
+        new_shape[unfolded_axis] = -1
+        # Warning! if the data is not contigous it will make a copy!!
+        data = self.data.reshape(new_shape)
+        for i in xrange(data.shape[unfolded_axis]):
+            getitem = [0] * len(data.shape)
+            for axis in axes:
+                getitem[axis] = slice(None)
+            getitem[unfolded_axis] = i
+            yield(data[getitem])
 
     @auto_replot
-    def sum(self, axis, return_signal = False):
+    def sum(self, axis, return_signal=False):
         """Sum the data over the specify axis
 
         Parameters
@@ -647,6 +681,51 @@ reconstruction created using either get_decomposition_model or get_bss_model met
         s.axes_manager.set_signal_dimension()
         if return_signal is True:
             return s
+            
+    def diff(self, axis, order=1, return_signal=False):
+        """Differentiate the data over the specify axis
+
+        Parameters
+        ----------
+        axis: int
+            The axis over which the operation will be performed
+        order: the order of the derivative
+        return_signal : bool
+            If False the operation will be performed on the current object. If
+            True, the current object will not be modified and the operation
+            will be performed in a new signal object that will be returned.
+
+        Returns
+        -------
+        Depending on the value of the return_signal keyword, nothing or a
+        signal instance
+
+        See also
+        --------
+        mean, sum
+
+        Usage
+        -----
+        >>> import numpy as np
+        >>> s = Signal({'data' : np.random.random((64,64,1024))})
+        >>> s.data.shape
+        (64,64,1024)
+        >>> s.diff(-1)
+        >>> s.data.shape
+        (64,64,1023)
+        # If we just want to plot the result of the operation
+        s.diff(-1, True).plot()
+        """
+        if return_signal is True:
+            s = self.deepcopy()
+        else:
+            s = self
+        s.data = np.diff(s.data,order,axis)
+        axis = s.axes_manager.axes[axis]
+        axis.offset = axis.axis[:2].mean()
+        s.get_dimensions_from_data()
+        if return_signal is True:
+            return s
 
     def mean(self, axis, return_signal = False):
         """Average the data over the specify axis
@@ -713,7 +792,7 @@ reconstruction created using either get_decomposition_model or get_bss_model met
         -------
         >>> import numpy as np
         >>> from hyperspy.signals.spectrum import Spectrum
-        >>> s = Spectrum({'data' : np.array([1,2,3,4,5])})
+        >>> s = signals.Spectrum({'data' : np.array([1,2,3,4,5])})
         >>> s.data
         array([1, 2, 3, 4, 5])
         >>> s.change_dtype('float')
@@ -729,7 +808,7 @@ reconstruction created using either get_decomposition_model or get_bss_model met
                                 same_window=None, comp_label='PC', 
                                 img_data=None,
                                 plot_shifts=True, plot_char=4, 
-                                cmap=plt.cm.jet, quiver_color='white',
+                                cmap=plt.cm.gray, quiver_color='white',
                                 vector_scale=1,
                                 per_row=3,ax=None):
         """Plot components from PCA or ICA, or peak characteristics
@@ -857,7 +936,7 @@ reconstruction created using either get_decomposition_model or get_bss_model met
     def _plot_loadings(self, loadings, comp_ids=None, calibrate=True,
                      same_window=None, comp_label=None, 
                      with_factors=False, factors=None,
-                     cmap=plt.cm.jet, no_nans=False, per_row=3):
+                     cmap=plt.cm.gray, no_nans=False, per_row=3):
         if same_window is None:
             same_window = preferences.MachineLearning.same_window
         if comp_ids is None:
@@ -938,7 +1017,7 @@ reconstruction created using either get_decomposition_model or get_bss_model met
                         factor_prefix=None,
                         factor_format=None,
                         comp_label=None,
-                        cmap=plt.cm.jet,
+                        cmap=plt.cm.gray,
                         plot_shifts=True,
                         plot_char=4,
                         img_data=None,
@@ -971,29 +1050,31 @@ reconstruction created using either get_decomposition_model or get_bss_model met
         if save_figures is True:
             plt.ioff()
             fac_plots=self._plot_factors_or_pchars(factors,
-                                                   comp_ids=comp_ids, 
-                                                   same_window=same_window,
-                                                   comp_label=comp_label, 
-                                                   img_data=img_data,
-                                                   plot_shifts=plot_shifts,
-                                                   plot_char=plot_char, 
-                                                   cmap=cmap,
-                                                   per_row=per_row,
-                                                   quiver_color=quiver_color,
-                                                   vector_scale=vector_scale)
+                                           comp_ids=comp_ids, 
+                                           same_window=same_window,
+                                           comp_label=comp_label, 
+                                           img_data=img_data,
+                                           plot_shifts=plot_shifts,
+                                           plot_char=plot_char, 
+                                           cmap=cmap,
+                                           per_row=per_row,
+                                           quiver_color=quiver_color,
+                                           vector_scale=vector_scale)
             for idx in xrange(len(comp_ids)):
-                filename = '%s_%02i.%s' % (factor_prefix, comp_ids[idx])
+                filename = '%s_%02i.%s' % (factor_prefix, comp_ids[idx],
+                save_figures_format)
                 if folder is not None:
                     filename = os.path.join(folder, filename)
                 ensure_directory(filename)
-                fac_plots[idx].savefig(filename, save_figures_format, dpi=600)
+                fac_plots[idx].savefig(filename, save_figures_format,
+                    dpi=600)
             plt.ion()
             
         elif multiple_files is False:
             if self.axes_manager.signal_dimension==2:
                 # factor images
                 axes_dicts=[]
-                axes=self.axes_manager._slicing_axes
+                axes=self.axes_manager.signal_axes
                 shape=(axes[1].size,axes[0].size)
                 factor_data=np.rollaxis(
                         factors.reshape((shape[0],shape[1],-1)),2)
@@ -1014,7 +1095,7 @@ reconstruction created using either get_decomposition_model or get_bss_model met
             elif self.axes_manager.signal_dimension==1:
                 axes=[]
                 axes.append(
-                self.axes_manager._slicing_axes[0].get_axis_dictionary())
+                self.axes_manager.signal_axes[0].get_axis_dictionary())
                 axes[0]['index_in_array']=1
                   
 
@@ -1038,7 +1119,7 @@ reconstruction created using either get_decomposition_model or get_bss_model met
         else: # Separate files
             if self.axes_manager.signal_dimension == 1:
             
-                axis_dict = self.axes_manager._slicing_axes[0].\
+                axis_dict = self.axes_manager.signal_axes[0].\
                     get_axis_dictionary()
                 axis_dict['index_in_array']=0
                 for dim,index in zip(comp_ids,range(len(comp_ids))):
@@ -1053,7 +1134,7 @@ reconstruction created using either get_decomposition_model or get_bss_model met
                     s.save(filename)
                     
             if self.axes_manager.signal_dimension == 2:
-                axes = self.axes_manager._slicing_axes
+                axes = self.axes_manager.signal_axes
                 axes_dicts=[]
                 axes_dicts.append(axes[0].get_axis_dictionary())
                 axes_dicts.append(axes[1].get_axis_dictionary())
@@ -1130,7 +1211,7 @@ reconstruction created using either get_decomposition_model or get_bss_model met
         elif multiple_files is False:
             if self.axes_manager.navigation_dimension==2:
                 axes_dicts=[]
-                axes=self.axes_manager._non_slicing_axes
+                axes=self.axes_manager.navigation_axes
                 shape=(axes[1].size,axes[0].size)
                 loading_data=loadings.reshape((-1,shape[0],shape[1]))
                 axes_dicts.append(axes[0].get_axis_dictionary())
@@ -1150,7 +1231,7 @@ reconstruction created using either get_decomposition_model or get_bss_model met
                                 self.mapped_parameters.title),
                             }})
             elif self.axes_manager.navigation_dimension==1:
-                cal_axis=self.axes_manager._non_slicing_axes[0].\
+                cal_axis=self.axes_manager.navigation_axes[0].\
                     get_axis_dictionary()
                 cal_axis['index_in_array']=1
                 axes=[]
@@ -1172,7 +1253,7 @@ reconstruction created using either get_decomposition_model or get_bss_model met
             s.save(filename)
         else: # Separate files
             if self.axes_manager.navigation_dimension == 1:
-                axis_dict = self.axes_manager._non_slicing_axes[0].\
+                axis_dict = self.axes_manager.navigation_axes[0].\
                     get_axis_dictionary()
                 axis_dict['index_in_array']=0
                 for dim,index in zip(comp_ids,range(len(comp_ids))):
@@ -1184,7 +1265,7 @@ reconstruction created using either get_decomposition_model or get_bss_model met
                     s.save(filename)
             elif self.axes_manager.navigation_dimension == 2:
                 axes_dicts=[]
-                axes=self.axes_manager._non_slicing_axes
+                axes=self.axes_manager.navigation_axes
                 shape=(axes[0].size, axes[1].size)
                 loading_data=loadings.reshape((-1,shape[0],shape[1]))
                 axes_dicts.append(axes[0].get_axis_dictionary())
